using System;
using System.Collections.Generic;
using Articy.Unity;
using Articy.Unity.Interfaces;
using UnityEngine;
using UnityEngine.UIElements;


public class WorldSpaceDialogueBox : MonoBehaviour {
    [SerializeField] private VisualTreeAsset visualTree;
    [SerializeField] private StyleSheet styleSheet;
    [SerializeField] private Katas.Experimental.WorldSpaceUIDocument worldSpaceUIDocument;
    [SerializeField] private string defaultButtonText = "Continue";
    private UIDocument uiDocument;
    private VisualElement root;
    private MeshRenderer componentRenderer;

    public Button[] buttons;


    public void Awake() {
        worldSpaceUIDocument.onUIDocumentReady.AddListener(SetUIDocument);
        worldSpaceUIDocument.onUIDocumentReady.AddListener(FindRenderer);
    }

    private void FindRenderer() {
        componentRenderer = GetComponent<MeshRenderer>();
    }

    private void SetUIDocument() {
        uiDocument = gameObject.GetComponent<UIDocument>();
        root = uiDocument.rootVisualElement;
        VisualElement tree = visualTree.CloneTree();
        tree.styleSheets.Add(styleSheet);
        root.Add(tree);
        buttons = root.Query<Button>().ToList().ToArray();
        ClearUI();

        for(var i = 0; i < buttons.Length; i++) {
            var index = i;
            buttons[i].RegisterCallback<ClickEvent>(_ =>
                gameObject.GetComponent<ArticyStoryline>().ApplyTransition(index));
        }
    }

    public void ClearUI() {
        SetActor(string.Empty);
        SetText(string.Empty);
        foreach(var button in buttons) {
            button.style.display = DisplayStyle.None;
        }
    }

    public void SetActor(string actor) {
        root.Q<Label>("Actor").text = actor;
    }

    public void SetActorFontSize(float size) {
        root.Q<Label>("Actor").style.fontSize = size;
    }

    public void SetText(string text) {
        root.Q<TextElement>("Dialogue").text = text;
    }

    public void SetTextFontSize(float size) {
        root.Q<TextElement>("Dialogue").style.fontSize = size; 
    }

    public void SetTransitions(IList<Branch> branches) {
        for(var i = 0; i < buttons.Length; i++) {
            buttons[i].text = string.Empty;

            if(i < branches.Count) {
                buttons[i].style.display = DisplayStyle.Flex;

                var menuText = branches[i].Target as IObjectWithMenuText;
                if(menuText != null && menuText.MenuText != string.Empty) {
                    buttons[i].text = menuText.MenuText;
                }
                else {
                    buttons[i].text = defaultButtonText;
                }
            }
            else {
                buttons[i].style.display = DisplayStyle.None;
            }
        }
    }
    
    public void ToggleVisibility() {
        if(componentRenderer != null) {
            componentRenderer.enabled = !componentRenderer.enabled;
        }
    }
}



